@@ -5,6 +5,8 @@
 # Copyright (c) 2013 Sai Suman Prayaga <suman.sai14@gmail.com>
 # Copyright (c) 2013 Shivani Poddar <shivani.poddar92@gmail.com>
 # Copyright (c) 2013 Guillaume Quintard <guillaume.quintard@gmail.com>
+# Copyright (c) 2014 Cedric Bellegarde <gnumdk@gmail.com>
+# Copyright (C) 2010 Jonathan Matthew (replay gain code)
 #
 # GNOME Music is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -37,6 +39,8 @@ from gettext import gettext as _
 from random import randint
 from queue import LifoQueue
 from gnomemusic.albumArtCache import AlbumArtCache
+from gnomemusic.playlists import Playlists
+playlists = Playlists.get_default()
 
 from gnomemusic import log
 import logging
@@ -86,7 +90,8 @@ class Player(GObject.GObject):
         self.currentTrack = None
         self._lastState = Gst.State.PAUSED
         self.cache = AlbumArtCache.get_default()
-        self._symbolicIcon = self.cache.get_default_icon(ART_SIZE, ART_SIZE)
+        self._noArtworkIcon = self.cache.get_default_icon(ART_SIZE, ART_SIZE)
+        self._loadingIcon = self.cache.get_default_icon(ART_SIZE, ART_SIZE, True)
 
         Gst.init(None)
 
@@ -98,10 +103,14 @@ class Player(GObject.GObject):
         self.player = Gst.ElementFactory.make('playbin', 'player')
         self.bus = self.player.get_bus()
         self.bus.add_signal_watch()
+        self.setup_replaygain()
 
         self._settings = Gio.Settings.new('org.gnome.Music')
-        self._settings.connect('changed::repeat', self._on_settings_changed)
+        self._settings.connect('changed::repeat', self._on_repeat_setting_changed)
+        self._settings.connect('changed::replaygain', self._on_replaygain_setting_changed)
         self.repeat = self._settings.get_enum('repeat')
+        self.replaygain = self._settings.get_value('replaygain') is not None
+        self.toggle_replaygain(self.replaygain)
 
         self.bus.connect('message::state-changed', self._on_bus_state_changed)
         self.bus.connect('message::error', self._onBusError)
@@ -111,6 +120,46 @@ class Player(GObject.GObject):
         self.playlist_insert_handler = 0
         self.playlist_delete_handler = 0
 
+    @log
+    def _on_replaygain_setting_changed(self, settings, value):
+        self.replaygain = settings.get_value('replaygain') is not None
+        self.toggle_replaygain(self.replaygain)
+
+    @log
+    def setup_replaygain(self):
+        """
+        Set up replaygain
+        See https://github.com/gnumdk/lollypop/commit/429383c3742e631b34937d8987d780edc52303c0
+        """
+        self._rgfilter = Gst.ElementFactory.make("bin", "bin")
+        self._rg_audioconvert1 = Gst.ElementFactory.make("audioconvert", "audioconvert")
+        self._rg_audioconvert2 = Gst.ElementFactory.make("audioconvert", "audioconvert2")
+        self._rgvolume = Gst.ElementFactory.make("rgvolume", "rgvolume")
+        self._rglimiter = Gst.ElementFactory.make("rglimiter", "rglimiter")
+        self._rg_audiosink = Gst.ElementFactory.make("autoaudiosink", "autoaudiosink")
+        if not self._rgfilter or not self._rg_audioconvert1 or not self._rg_audioconvert2 \
+           or not self._rgvolume or not self._rglimiter or not self._rg_audiosink:
+            logger.debug("Replay Gain is not available")
+            return
+        self._rgvolume.props.pre_amp = 0.0
+        self._rgfilter.add(self._rgvolume)
+        self._rgfilter.add(self._rg_audioconvert1)
+        self._rgfilter.add(self._rg_audioconvert2)
+        self._rgfilter.add(self._rglimiter)
+        self._rgfilter.add(self._rg_audiosink)
+        self._rg_audioconvert1.link(self._rgvolume)
+        self._rgvolume.link(self._rg_audioconvert2)
+        self._rgvolume.link(self._rglimiter)
+        self._rg_audioconvert2.link(self._rg_audiosink)
+        self._rgfilter.add_pad(Gst.GhostPad.new("sink", self._rg_audioconvert1.get_static_pad("sink")))
+
+    @log
+    def toggle_replaygain(self, state=False):
+        if state and self._rgfilter:
+            self.player.set_property("audio-sink", self._rgfilter)
+        else:
+            self.player.set_property("audio-sink", None)
+
     def discover_item(self, item, callback, data=None):
         url = item.get_url()
         if not url:
@@ -118,7 +167,7 @@ class Player(GObject.GObject):
             return
 
         if not url.startswith("file://"):
-            logger.debug("Skipping discovery of %s as a remote url" % url)
+            logger.debug("Skipping discovery of %s as not a local file" % url)
             return
 
         obj = (callback, data)
@@ -144,7 +193,7 @@ class Player(GObject.GObject):
             return
 
     @log
-    def _on_settings_changed(self, settings, value):
+    def _on_repeat_setting_changed(self, settings, value):
         self.repeat = settings.get_enum('repeat')
         self._sync_prev_next()
         self._sync_repeat_image()
@@ -405,7 +454,7 @@ class Player(GObject.GObject):
         except:
             pass
 
-        self.coverImg.set_from_pixbuf(self._symbolicIcon)
+        self.coverImg.set_from_pixbuf(self._noArtworkIcon)
         self.cache.lookup(
             media, ART_SIZE, ART_SIZE, self._on_cache_lookup, None, artist, album)
 
@@ -620,6 +669,8 @@ class Player(GObject.GObject):
     @log
     def _set_duration(self, duration):
         self.duration = duration
+        self.played_seconds = 0
+        self.scrobbled = False
         self.progressScale.set_range(0.0, duration * 60)
 
     @log
@@ -627,6 +678,16 @@ class Player(GObject.GObject):
         position = self.player.query_position(Gst.Format.TIME)[1] / 1000000000
         if position > 0:
             self.progressScale.set_value(position * 60)
+            self.played_seconds += 1
+            try:
+                percentage = self.played_seconds / self.duration
+                if not self.scrobbled and percentage > 0.4:
+                    just_played_url = self.get_current_media().get_url()
+                    self.scrobbled = True
+                    playlists.update_playcount(just_played_url)
+                    playlists.update_last_played(just_played_url)
+            except Exception as e:
+                logger.warn("Error: %s, %s" % (e.__class__, e))
         return True
 
     @log
