@@ -32,14 +32,12 @@
 
 
 from gi.repository import Gtk
-from gi.repository import Gdk
 from gi.repository import GObject
 from gi.repository import Gd
 from gi.repository import Grl
 from gi.repository import Pango
 from gi.repository import GLib
 from gi.repository import GdkPixbuf
-from gi.repository import Tracker
 
 from gettext import gettext as _, ngettext
 from gnomemusic.grilo import grilo
@@ -52,13 +50,6 @@ from gnomemusic import log
 import logging
 logger = logging.getLogger(__name__)
 
-try:
-    tracker = Tracker.SparqlConnection.get(None)
-except Exception as e:
-    from sys import exit
-    logger.error("Cannot connect to tracker, error '%s'\Exiting" % str(e))
-    exit(1)
-
 playlists = Playlists.get_default()
 
 
@@ -97,7 +88,6 @@ class ViewContainer(Gtk.Stack):
         )
         self.view.set_view_type(view_type)
         self.view.set_model(self._model)
-        self.vadjustment = self.view.get_vadjustment()
         box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
         box.pack_start(self.view, True, True, 0)
         if use_sidebar:
@@ -131,11 +121,10 @@ class ViewContainer(Gtk.Stack):
         self.add(self._grid)
 
         self.show_all()
+        self.view.hide()
         self._items = []
-        self._connect_view()
         self.cache = albumArtCache.get_default()
-        self._symbolicIcon = self.cache.get_default_icon(self._iconHeight,
-                                                         self._iconWidth)
+        self._loadingIcon = self.cache.get_default_icon(self._iconWidth, self._iconHeight, True)
 
         self._init = False
         grilo.connect('ready', self._on_grilo_ready)
@@ -212,57 +201,6 @@ class ViewContainer(Gtk.Stack):
     def _on_selection_mode_changed(self, widget, data=None):
         pass
 
-    @log
-    def _connect_view(self):
-        if self.view.get_view_type() == Gd.MainViewType.ICON:
-            self.view.connect_after('draw', self._on_view_draw)
-            self.view.add_events(Gdk.EventMask.EXPOSURE_MASK)
-
-    @log
-    def _on_view_draw(self, widget, cr):
-        vsbl_range = widget.get_children()[0].get_visible_range()
-        if not vsbl_range or self.old_vsbl_range == vsbl_range:
-            return
-        self.old_vsbl_range = vsbl_range
-        GLib.idle_add(self.on_scroll_event, self.view)
-
-    @log
-    def on_scroll_event(self, widget, event=None):
-        vsbl_range = widget.get_children()[0].get_visible_range()
-        if not vsbl_range:
-            return
-
-        def load_album_art_for(path):
-            try:
-                _iter = self._model.get_iter(path)
-                item = self._model.get_value(_iter, 5)
-                if not item:
-                    return
-                title = self._model.get_value(_iter, 2)
-                artist = self._model.get_value(_iter, 3)
-                thumbnail = self._model.get_value(_iter, 4)
-                if thumbnail == self._symbolicIcon:
-                    albumArtCache.get_default().lookup(
-                        item, self._iconWidth, self._iconHeight, self._on_lookup_ready,
-                        _iter, artist, title)
-            except Exception:
-                pass
-
-        # Load thumbnails
-        path = vsbl_range[0]
-        while path <= vsbl_range[1]:
-            load_album_art_for(path)
-            path.next()
-
-        # Add 10 more albums to avoid visible thumbnail loading
-        for i in range(0, 10):
-            try:
-                path.next()
-                load_album_art_for(path)
-            except ValueError:
-                # No such path
-                break
-
     @log
     def populate(self):
         print('populate')
@@ -270,19 +208,24 @@ class ViewContainer(Gtk.Stack):
     @log
     def _on_discovered(self, info, error, _iter):
         if error:
-            print("Info %s: error: %s" % (info, error))
-            self._model.set(_iter, [8, 10], [self.errorIconName, True])
+            try:
+                logger.warn("File will be skipped: %s\n%s" % (error.message, info.get_uri()))
+                self._model.set(_iter, [8, 10], [self.errorIconName, True])
+            except Exception:
+                pass
 
     @log
     def _add_item(self, source, param, item, remaining=0, data=None):
         if not item:
+            if remaining == 0:
+                self.view.show()
             return
         self._offset += 1
         artist = item.get_string(Grl.METADATA_KEY_ARTIST)\
             or item.get_author()\
             or _("Unknown Artist")
         title = albumArtCache.get_media_title(item)
-        item.set_title(title)
+        # item.set_title(title)
 
         def add_new_item():
             _iter = self._model.append(None)
@@ -291,13 +234,15 @@ class ViewContainer(Gtk.Stack):
                 try:
                     self.player.discoverer.discover_uri(item.get_url())
                 except:
-                    print('failed to discover url ' + item.get_url())
+                    logger.warn('failed to discover url ' + item.get_url())
                     icon_name = self.errorIconName
             self._model.set(_iter,
                             [0, 1, 2, 3, 4, 5, 7, 8, 9, 10],
                             [str(item.get_id()), '', title,
-                             artist, self._symbolicIcon, item,
+                             artist, self._loadingIcon, item,
                              0, icon_name, False, icon_name == self.errorIconName])
+            self.cache.lookup(item, self._iconWidth, self._iconHeight, self._on_lookup_ready,
+                              _iter, artist, title)
         GLib.idle_add(add_new_item)
 
     @log
@@ -366,7 +311,10 @@ class Albums(ViewContainer):
 
     @log
     def _on_item_activated(self, widget, id, path):
-        _iter = self._model.get_iter(path)
+        try:
+            _iter = self._model.get_iter(path)
+        except TypeError:
+            return
         title = self._model.get_value(_iter, 2)
         artist = self._model.get_value(_iter, 3)
         item = self._model.get_value(_iter, 5)
@@ -429,9 +377,6 @@ class Songs(ViewContainer):
             .add_class('songs-list')
         self._iconHeight = 32
         self._iconWidth = 32
-        self.cache = albumArtCache.get_default()
-        self._symbolicIcon = self.cache.get_default_icon(self._iconHeight,
-                                                         self._iconWidth)
         self._add_list_renderers()
         self.player = player
         self.player.connect('playlist-item-changed', self.update_model)
@@ -451,7 +396,10 @@ class Songs(ViewContainer):
 
     @log
     def _on_item_activated(self, widget, id, path):
-        _iter = self._model.get_iter(path)
+        try:
+            _iter = self._model.get_iter(path)
+        except TypeError:
+            return
         if self._model.get_value(_iter, 8) != self.errorIconName:
             self.player.set_playlist('Songs', None, self._model, _iter, 5)
             self.player.set_playing(True)
@@ -472,6 +420,8 @@ class Songs(ViewContainer):
 
     def _add_item(self, source, param, item, remaining=0, data=None):
         if not item:
+            if remaining == 0:
+                self.view.show()
             return
         self._offset += 1
         item.set_title(albumArtCache.get_media_title(item))
@@ -616,6 +566,7 @@ class Artists (ViewContainer):
             self.view.get_generic_view().get_style_context().\
                 add_class('artist-panel-white')
         self.show_all()
+        self.view.hide()
 
     @log
     def _on_changes_pending(self, data=None):
@@ -637,6 +588,9 @@ class Artists (ViewContainer):
             selection.select_path(self._model.get_path(self._allIter))
         self._init = True
         self.populate()
+
+    @log
+    def add_all_artists_entry(self):
         self.view.emit('item-activated', '0',
                        self._model.get_path(self._allIter))
 
@@ -661,7 +615,10 @@ class Artists (ViewContainer):
 
     @log
     def _on_item_activated(self, widget, item_id, path):
-        _iter = self._model.get_iter(path)
+        try:
+            _iter = self._model.get_iter(path)
+        except TypeError:
+            return
         self._last_selection = _iter
         artist = self._model.get_value(_iter, 2)
         albums = self._artists[artist.casefold()]['albums']
@@ -707,6 +664,9 @@ class Artists (ViewContainer):
     @log
     def _add_item(self, source, param, item, remaining=0, data=None):
         if item is None:
+            if remaining == 0:
+                self.add_all_artists_entry()
+                self.view.show()
             return
         self._offset += 1
         artist = item.get_string(Grl.METADATA_KEY_ARTIST)\
@@ -828,6 +788,7 @@ class Playlist(ViewContainer):
             GObject.TYPE_BOOLEAN,
             GObject.TYPE_BOOLEAN
         )
+
         self.playlists_sidebar.set_view_type(Gd.MainViewType.LIST)
         self.playlists_sidebar.set_model(self.playlists_model)
         self.playlists_sidebar.set_hexpand(False)
@@ -850,11 +811,15 @@ class Playlist(ViewContainer):
         self.iter_to_clean = None
         self.iter_to_clean_model = None
         self.current_playlist = None
+        self.pl_todelete = None
+        self.really_delete = True
         self.songs_count = 0
+        self.window = window
         self._update_songs_count()
         self.player = player
         self.player.connect('playlist-item-changed', self.update_model)
         playlists.connect('playlist-created', self._on_playlist_created)
+        playlists.connect('playlist-updated', self.on_playlist_update)
         playlists.connect('song-added-to-playlist', self._on_song_added_to_playlist)
         playlists.connect('song-removed-from-playlist', self._on_song_removed_from_playlist)
         self.show_all()
@@ -1001,7 +966,10 @@ class Playlist(ViewContainer):
 
     @log
     def _on_item_activated(self, widget, id, path):
-        _iter = self._model.get_iter(path)
+        try:
+            _iter = self._model.get_iter(path)
+        except TypeError:
+            return
         if self._model.get_value(_iter, 8) != self.errorIconName:
             self.player.set_playlist(
                 'Playlist', self.current_playlist.get_id(),
@@ -1009,8 +977,22 @@ class Playlist(ViewContainer):
             )
             self.player.set_playing(True)
 
+    @log
+    def on_playlist_update(self, widget, playlist_id):
+        _iter = self.playlists_model.get_iter_first()
+        while _iter:
+            playlist = self.playlists_model.get_value(_iter, 5)
+            if str(playlist_id) == playlist.get_id() and self.current_playlist == playlist:
+                path = self.playlists_model.get_path(_iter)
+                self._on_playlist_activated(None, None, path, skip_cache=True)
+                selection = self.playlists_sidebar.get_generic_view().get_selection()
+                selection.select_iter(_iter)
+                break
+            _iter = self.playlists_model.iter_next(_iter)
+
     @log
     def activate_playlist(self, playlist_id):
+
         def find_and_activate_playlist():
             for playlist in self.playlists_model:
                 if playlist[5].get_id() == playlist_id:
@@ -1051,7 +1033,7 @@ class Playlist(ViewContainer):
             self._populate()
 
     @log
-    def _on_playlist_activated(self, widget, item_id, path):
+    def _on_playlist_activated(self, widget, item_id, path, skip_cache=False):
         _iter = self.playlists_model.get_iter(path)
         playlist_name = self.playlists_model.get_value(_iter, 2)
         playlist = self.playlists_model.get_value(_iter, 5)
@@ -1065,7 +1047,7 @@ class Playlist(ViewContainer):
         # if the active queue has been set by this playlist,
         # use it as model, otherwise build the liststore
         cached_playlist = self.player.running_playlist('Playlist', playlist_name)
-        if cached_playlist:
+        if cached_playlist and not skip_cache:
             self._model = cached_playlist
             currentTrack = self.player.playlist.get_iter(self.player.currentTrack.get_path())
             self.update_model(self.player, cached_playlist,
@@ -1139,7 +1121,7 @@ class Playlist(ViewContainer):
                        self._model.get_path(_iter))
 
     @log
-    def _on_delete_activate(self, menuitem, data=None):
+    def delete_selected_playlist(self):
         self._model.clear()
         _iter = self.playlists_sidebar.get_generic_view().get_selection().get_selected()[1]
         if not _iter:
@@ -1154,9 +1136,18 @@ class Playlist(ViewContainer):
                                         self.playlists_model.get_path(iter_next))
 
         playlist = self.playlists_model.get_value(_iter, 5)
-        playlists.delete_playlist(playlist)
+        self.pl_todelete = playlist
         self.playlists_model.remove(_iter)
 
+    @log
+    def undo_playlist(self):
+        self._add_playlist_item_to_model(self.pl_todelete)
+
+    @log
+    def _on_delete_activate(self, menuitem, data=None):
+        self.window._init_notification()
+        self.delete_selected_playlist()
+
     @log
     def _on_playlist_created(self, playlists, item):
         self._add_playlist_item_to_model(item)
@@ -1249,9 +1240,8 @@ class Search(ViewContainer):
         self.iter_to_clean = None
         self._iconHeight = 48
         self._iconWidth = 48
-        self.cache = albumArtCache.get_default()
-        self._symbolicIcon = self.cache.get_default_icon(self._iconHeight,
-                                                         self._iconWidth)
+        self._loadingIcon = self.cache.get_default_icon(self._iconWidth, self._iconHeight, True)
+        self._noAlbumArtIcon = self.cache.get_default_icon(self._iconWidth, self._iconHeight, False)
         self._add_list_renderers()
         self.player = player
         self.head_iters = [None, None, None, None]
@@ -1280,7 +1270,10 @@ class Search(ViewContainer):
 
     @log
     def _on_item_activated(self, widget, id, path):
-        child_path = self.filter_model.convert_path_to_child_path(path)
+        try:
+            child_path = self.filter_model.convert_path_to_child_path(path)
+        except TypeError:
+            return
         _iter = self._model.get_iter(child_path)
         if self._model[_iter][11] == 'album':
             title = self._model.get_value(_iter, 2)
@@ -1353,6 +1346,9 @@ class Search(ViewContainer):
         if data is None:
             return
 
+        if remaining == 0:
+            self.view.show()
+
         model, category = data
         if not item or model != self._model:
             return
@@ -1371,12 +1367,21 @@ class Search(ViewContainer):
             pass
 
         _iter = None
-        if category == 'album' or category == 'song':
+        if category == 'album':
             _iter = self._model.insert_with_values(
                 self.head_iters[group], -1,
                 [0, 2, 3, 4, 5, 8, 9, 10, 11],
                 [str(item.get_id()), title, artist,
-                 self._symbolicIcon, item, self.nowPlayingIconName,
+                 self._loadingIcon, item, self.nowPlayingIconName,
+                 False, False, category])
+            self.cache.lookup(item, self._iconWidth, self._iconHeight, self._on_lookup_ready,
+                              _iter, artist, title)
+        elif category == 'song':
+            _iter = self._model.insert_with_values(
+                self.head_iters[group], -1,
+                [0, 2, 3, 4, 5, 8, 9, 10, 11],
+                [str(item.get_id()), title, artist,
+                 self._noAlbumArtIcon, item, self.nowPlayingIconName,
                  False, False, category])
         else:
             if not artist.casefold() in self._artists:
@@ -1384,17 +1389,14 @@ class Search(ViewContainer):
                     self.head_iters[group], -1,
                     [0, 2, 4, 5, 8, 9, 10, 11],
                     [str(item.get_id()), artist,
-                     self._symbolicIcon, item, self.nowPlayingIconName,
+                     self._loadingIcon, item, self.nowPlayingIconName,
                      False, False, category])
+                self.cache.lookup(item, self._iconWidth, self._iconHeight, self._on_lookup_ready,
+                                  _iter, artist, title)
                 self._artists[artist.casefold()] = {'iter': _iter, 'albums': []}
 
             self._artists[artist.casefold()]['albums'].append(item)
 
-        if _iter:
-            albumArtCache.get_default().lookup(
-                item, self._iconWidth, self._iconHeight, self._on_lookup_ready,
-                _iter, artist, title)
-
         if category == 'song':
             self.player.discover_item(item, self._on_discovered, _iter)
 
@@ -1530,6 +1532,10 @@ class Search(ViewContainer):
     def _filter_visible_func(self, model, _iter, data=None):
         return model.iter_parent(_iter) is not None or model.iter_has_child(_iter)
 
+    @log
+    def _on_grilo_ready(self, data=None):
+        playlists.fetch_or_create_static_playlists()
+
     @log
     def set_search_text(self, search_term, fields_filter):
         query_matcher = {
