@@ -33,7 +33,6 @@
 
 from gi.repository import Gtk, Gdk, Gd, GLib, GObject, Pango, Gio
 from gi.repository import GdkPixbuf, Grl
-from gi.repository import Tracker
 from gettext import gettext as _, ngettext
 from gnomemusic.grilo import grilo
 from gnomemusic.albumArtCache import AlbumArtCache
@@ -42,25 +41,24 @@ from gnomemusic import log
 import logging
 logger = logging.getLogger(__name__)
 
-playlist = Playlists.get_default()
-
-try:
-    tracker = Tracker.SparqlConnection.get(None)
-except Exception as e:
-    from sys import exit
-    logger.error("Cannot connect to tracker, error '%s'\Exiting" % str(e))
-    exit(1)
-
 ALBUM_ART_CACHE = AlbumArtCache.get_default()
 NOW_PLAYING_ICON_NAME = 'media-playback-start-symbolic'
 ERROR_ICON_NAME = 'dialog-error-symbolic'
 
+try:
+    settings = Gio.Settings.new('org.gnome.Music')
+    MAX_TITLE_WIDTH = settings.get_int('max-width-chars')
+except Exception as e:
+    MAX_TITLE_WIDTH = 20
+    logger.error("Error on setting widget max-width-chars: %s" % str(e))
+
 
 class AlbumWidget(Gtk.EventBox):
 
     tracks = []
     duration = 0
-    symbolicIcon = ALBUM_ART_CACHE.get_default_icon(256, 256)
+    loadingIcon = ALBUM_ART_CACHE.get_default_icon(256, 256, True)
+    noArtworkIcon = ALBUM_ART_CACHE.get_default_icon(256, 256, False)
 
     @log
     def __init__(self, player):
@@ -179,7 +177,7 @@ class AlbumWidget(Gtk.EventBox):
         real_artist = item.get_string(Grl.METADATA_KEY_ARTIST)\
             or item.get_author()\
             or _("Unknown Artist")
-        self.ui.get_object('cover').set_from_pixbuf(self.symbolicIcon)
+        self.ui.get_object('cover').set_from_pixbuf(self.loadingIcon)
         ALBUM_ART_CACHE.lookup(item, 256, 256, self._on_look_up, None, real_artist, album)
 
         # if the active queue has been set by self album,
@@ -272,10 +270,11 @@ class AlbumWidget(Gtk.EventBox):
     @log
     def _on_look_up(self, pixbuf, path, data=None):
         _iter = self.iterToClean
-        if pixbuf:
-            self.ui.get_object('cover').set_from_pixbuf(pixbuf)
-            if _iter:
-                self.model.set(_iter, [4], [pixbuf])
+        if not pixbuf:
+            pixbuf = self.noArtworkIcon
+        self.ui.get_object('cover').set_from_pixbuf(pixbuf)
+        if _iter:
+            self.model.set(_iter, [4], [pixbuf])
 
     @log
     def update_model(self, player, playlist, currentIter):
@@ -353,14 +352,21 @@ class ArtistAlbums(Gtk.Box):
         self._songsGridSizeGroup = Gtk.SizeGroup.new(Gtk.SizeGroupMode.HORIZONTAL)
         self.pack_start(self._scrolledWindow, True, True, 0)
 
+        self.hide()
+
         for album in albums:
-            self.add_album(album)
+            is_last_album = False
+            if album == albums[-1]:
+                is_last_album = True
+            self.add_album(album, is_last_album)
 
-        self.show_all()
         self.player.connect('playlist-item-changed', self.update_model)
 
+    def _on_last_album_displayed(self, data=None):
+        self.show_all()
+
     @log
-    def add_album(self, album):
+    def add_album(self, album, is_last_album=False):
         widget = ArtistAlbumWidget(
             self.artist, album, self.player, self.model,
             self.header_bar, self.selectionModeAllowed
@@ -370,6 +376,9 @@ class ArtistAlbums(Gtk.Box):
         self._albumBox.pack_start(widget, False, False, 0)
         self.widgets.append(widget)
 
+        if is_last_album:
+            widget.connect('tracks-loaded', self._on_last_album_displayed)
+
     @log
     def update_model(self, player, playlist, currentIter):
         # this is not our playlist, return
@@ -453,6 +462,7 @@ class AllArtistsAlbums(ArtistAlbums):
                               header_bar, selection_toolbar, selectionModeAllowed)
         self._offset = 0
         self._populate()
+        self.show()
 
     @log
     def _populate(self, data=None):
@@ -465,11 +475,17 @@ class AllArtistsAlbums(ArtistAlbums):
         if item:
             self._offset += 1
             self.add_album(item)
+            self.show_all()
 
 
 class ArtistAlbumWidget(Gtk.Box):
 
-    pixbuf = AlbumArtCache.get_default().get_default_icon(128, 128)
+    __gsignals__ = {
+        'tracks-loaded': (GObject.SIGNAL_RUN_FIRST, None, ()),
+    }
+
+    loadingIcon = AlbumArtCache.get_default().get_default_icon(128, 128, True)
+    noArtworkIcon = ALBUM_ART_CACHE.get_default_icon(128, 128, False)
 
     @log
     def __init__(self, artist, album, player, model, header_bar, selectionModeAllowed):
@@ -489,7 +505,7 @@ class ArtistAlbumWidget(Gtk.Box):
         GLib.idle_add(self._update_album_art)
 
         self.cover = self.ui.get_object('cover')
-        self.cover.set_from_pixbuf(self.pixbuf)
+        self.cover.set_from_pixbuf(self.loadingIcon)
         self.songsGrid = self.ui.get_object('grid1')
         self.ui.get_object('title').set_label(album.get_title())
         if album.get_creation_date():
@@ -498,16 +514,8 @@ class ArtistAlbumWidget(Gtk.Box):
                 str(album.get_creation_date().get_year())
             )
         self.tracks = []
-        GLib.idle_add(grilo.populate_album_songs, album, self.add_item)
+        grilo.populate_album_songs(album, self.add_item)
         self.pack_start(self.ui.get_object('ArtistAlbumWidget'), True, True, 0)
-        self.show_all()
-
-        try:
-            self.settings = Gio.Settings.new('org.gnome.Music')
-            self.max_title_width = self.settings.get_int('max-width-chars')
-        except Exception as e:
-            self.max_title_width = 20
-            logger.error("Error on setting widget max-width-chars: %s" % str(e))
 
     @log
     def _on_discovered(self, info, error, song_widget):
@@ -521,6 +529,10 @@ class ArtistAlbumWidget(Gtk.Box):
 
     @log
     def add_item(self, source, prefs, track, remaining, data=None):
+        if remaining == 0:
+            self.songsGrid.show_all()
+            self.emit("tracks-loaded")
+
         if track:
             self.tracks.append(track)
         else:
@@ -535,7 +547,7 @@ class ArtistAlbumWidget(Gtk.Box):
                 title = AlbumArtCache.get_media_title(track)
                 ui.get_object('title').set_text(title)
                 ui.get_object('title').set_alignment(0.0, 0.5)
-                ui.get_object('title').set_max_width_chars(self.max_title_width)
+                ui.get_object('title').set_max_width_chars(MAX_TITLE_WIDTH)
 
                 self.songsGrid.attach(
                     song_widget,
@@ -566,7 +578,6 @@ class ArtistAlbumWidget(Gtk.Box):
                 song_widget.can_be_played = True
                 song_widget.connect('button-release-event',
                                     self.track_selected)
-            self.songsGrid.show_all()
 
     @log
     def _update_album_art(self):
@@ -579,8 +590,9 @@ class ArtistAlbumWidget(Gtk.Box):
 
     @log
     def _get_album_cover(self, pixbuf, path, data=None):
-        if pixbuf:
-            self.cover.set_from_pixbuf(pixbuf)
+        if not pixbuf:
+            pixbuf = self.noArtworkIcon
+        self.cover.set_from_pixbuf(pixbuf)
 
     @log
     def track_selected(self, widget, event):
@@ -656,7 +668,8 @@ class PlaylistDialog():
         self._cancel_button.connect('clicked', self._on_cancel_button_clicked)
         self._select_button.connect('clicked', self._on_selection)
 
-        playlist.connect('playlist-created', self._on_playlist_created)
+        self.playlist = Playlists.get_default()
+        self.playlist.connect('playlist-created', self._on_playlist_created)
 
     @log
     def get_selected(self):
@@ -738,12 +751,13 @@ class PlaylistDialog():
     @log
     def _on_editing_done(self, editable, data=None):
         if editable.get_text() != '':
-            playlist.create_playlist(editable.get_text())
+            self.playlist.create_playlist(editable.get_text())
 
     @log
     def _on_playlist_created(self, playlists, item):
         new_iter = self._add_item_to_model(item)
-        self.view.set_cursor(self.model.get_path(new_iter),
-                             self.view.get_columns()[0], False)
-        self.view.row_activated(self.model.get_path(new_iter),
-                                self.view.get_columns()[0])
+        if self.view.get_columns():
+            self.view.set_cursor(self.model.get_path(new_iter),
+                                 self.view.get_columns()[0], False)
+            self.view.row_activated(self.model.get_path(new_iter),
+                                    self.view.get_columns()[0])
