@@ -31,6 +31,7 @@
 
 
 from gi.repository import Gtk, Gdk, Gio, GLib, Tracker
+from gi.repository import Gd
 from gettext import gettext as _, ngettext
 
 from gnomemusic.toolbar import Toolbar, ToolbarState
@@ -44,13 +45,13 @@ from gnomemusic import log
 import logging
 logger = logging.getLogger(__name__)
 
-playlist = Playlists.get_default()
 try:
     tracker = Tracker.SparqlConnection.get(None)
 except Exception as e:
     from sys import exit
     logger.error("Cannot connect to tracker, error '%s'\Exiting" % str(e))
     exit(1)
+playlist = Playlists.get_default(tracker)
 
 
 class Window(Gtk.ApplicationWindow):
@@ -89,9 +90,11 @@ class Window(Gtk.ApplicationWindow):
         if self.settings.get_value('window-maximized'):
             self.maximize()
 
+        self._setup_view()
+
         self.connect("window-state-event", self._on_window_state_event)
         self.connect("configure-event", self._on_configure_event)
-        self._setup_view()
+
         self.proxy = Gio.DBusProxy.new_sync(Gio.bus_get_sync(Gio.BusType.SESSION, None),
                                             Gio.DBusProxyFlags.NONE,
                                             None,
@@ -195,22 +198,27 @@ class Window(Gtk.ApplicationWindow):
         self._box.pack_start(self.player.actionbar, False, False, 0)
         self._box.pack_start(self.selection_toolbar.actionbar, False, False, 0)
         self.add(self._box)
-        count = 1
+        count = 0
         cursor = None
-        try:
-            cursor = tracker.query(Query.all_songs_count(), None)
-        except Exception as e:
-            logger.error("Tracker query crashed: %s" % e)
-            count = 0
 
-        if cursor is not None and cursor.next(None):
-            count = cursor.get_integer(0)
-        if count > 0:
-            self._switch_to_player_view()
-        # To revert to the No Music View when no songs are found
+        if Query.music_folder and Query.download_folder:
+            try:
+                cursor = tracker.query(Query.all_songs_count(), None)
+                if cursor is not None and cursor.next(None):
+                    count = cursor.get_integer(0)
+            except Exception as e:
+                logger.error("Tracker query crashed: %s" % e)
+                count = 0
+
+            if count > 0:
+                self._switch_to_player_view()
+            # To revert to the No Music View when no songs are found
+            else:
+                if self.toolbar._selectionMode is False:
+                    self._switch_to_empty_view()
         else:
-            if self.toolbar._selectionMode is False:
-                self._switch_to_empty_view()
+            # Revert to No Music view if XDG dirs are not set
+            self._switch_to_empty_view()
 
         self.toolbar._search_button.connect('toggled', self._on_search_toggled)
         self.toolbar.connect('selection-mode-changed', self._on_selection_mode_changed)
@@ -300,6 +308,38 @@ class Window(Gtk.ApplicationWindow):
         self.toolbar._selection_menu_label.set_text(_("Click on items to select them"))
         self._stack.get_visible_child().queue_draw()
 
+    @log
+    def _init_notification(self):
+        self.notification = Gd.Notification()
+        self.notification.set_timeout(20)
+
+        grid = Gtk.Grid()
+        grid.set_column_spacing(8)
+        self.notification.add(grid)
+
+        undo_button = Gtk.Button.new_with_mnemonic(_("_Undo"))
+        label = _("Playlist %s removed" %(
+            self.views[3].current_playlist.get_title()))
+        grid.add(Gtk.Label.new(label))
+        grid.add(undo_button)
+
+        self.notification.show_all()
+        self._overlay.add_overlay(self.notification)
+
+        self.notification.connect("dismissed", self._notification_dismissed)
+        undo_button.connect("clicked", self._undo_deletion)
+
+    @log
+    def _notification_dismissed(self, widget):
+        if self.views[3].really_delete:
+            Views.playlists.delete_playlist(self.views[3].pl_todelete)
+
+    @log
+    def _undo_deletion(self, widget):
+        self.views[3].really_delete = False
+        self.notification.dismiss()
+        self.views[3].undo_playlist()
+
     @log
     def _on_key_press(self, widget, event):
         modifiers = Gtk.accelerator_get_default_mod_mask()
@@ -317,6 +357,10 @@ class Window(Gtk.ApplicationWindow):
                     self.curr_view.set_visible_child(self.curr_view._grid)
                     self.toolbar.set_state(ToolbarState.MAIN)
         else:
+            if (event.keyval == Gdk.KEY_Delete):
+                if self._stack.get_visible_child() == self.views[3]:
+                    self._init_notification()
+                    self.views[3].delete_selected_playlist()
             # Close search bar after Esc is pressed
             if event.keyval == Gdk.KEY_Escape:
                 self.toolbar.searchbar.show_bar(False)
